/*******************************************************************************
 * Copyright (c) 2009 Ferenc Hechler - ferenc_hechler@users.sourceforge.net
 * 
 * This file is part of the Android Hotspot Login
 *
 * The Android Hotspot Login is free software;
 * you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option) any later version.
 * 
 * The Android Hotspot Login is distributed
 * in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the Android Hotspot Login;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *  
 *******************************************************************************/
package net.sf.andhsli.hotspotlogin;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.RemoteException;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

/**
 * This class provides a basic demonstration of how to write an Android
 * activity. Inside of its window, it places a single view: an EditText that
 * displays and edits some internal text.
 */
public class HotspotLoginActivity extends Activity {

	public final static String VERSION = "0.1.5";
	// remember to change res/raw/release_notes.txt

	public final static int	SPLASH_SCREEN = 1;

	private static final String TAG = "HSLI.gui";
	public final static String PREFS_NAME = "HotspotLoginPrefs";
	public final static String SETTINGPREFS_NAME = "HotspotLoginSettings3Prefs";
    public static final String EXTRA_SETTINGS = "net.sf.andhsli.extra.SETTINGS";
    public static final String EXTRA_LOGIN = "net.sf.andhsli.extra.LOGIN";
    public static final String EXTRA_PW = "net.sf.andhsli.extra.PW";
    

	public static final String TRANSMIT_SEED = "transMit.Seed";
	
    boolean serviceBound;

    private boolean valuesRestored;
    private boolean initializationPhase;
    
    private EditText mLogin;
    private EditText mPW;
	private TextView mStatus;
	private CheckBox mLockWifi;
	private CheckBox mCreateSSID;
	private CheckBox mBeep;
	private Spinner  mScanIntervall;
	private CheckBox mHideWaitingIcon;
	private CheckBox mAutostartService;
	private CheckBox mEnableLogCat;
	private String jsonParams;
	
	Button btStart;
	Button btStop;

	private Thread updateThread;
	
    private final static int MSG_SETSTATUS     = 1;
    private final static int MSG_RESTOREVALUES = 2;
    class UIHandler extends Handler {
    	@Override
    	public void handleMessage(Message msg) {
    		super.handleMessage(msg);
    		switch (msg.what) {
    		case(MSG_SETSTATUS): {
    			String st = (String)msg.obj;
    			mStatus.setText(st);
    			break;
    		}
    		case(MSG_RESTOREVALUES): {
    			restoreValues();
    	        Log.logEnabled = mEnableLogCat.isChecked();
    	        if (mAutostartService.isChecked()) {
    	        	// auto start service on app start
    	        	if (initializationPhase) {
    	        		initializationPhase = false;
    	        		StartServiceListener.onClick(null);
    	        	}
    	        }
    			break;
    		}
    		}
    	}
    	public void setStatus(String st) {
    		Message msg = Message.obtain(this, MSG_SETSTATUS, st);
    		sendMessage(msg);
    	}
    	public void restValues() {
    		Message msg = Message.obtain(this, MSG_RESTOREVALUES);
    		sendMessage(msg);
    	}
    }    
	private UIHandler uiHandler;
    
	public HotspotLoginActivity() {
    }

    /** Called with the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        uiHandler = new UIHandler(); 
        
		// create view from xml
        setContentView(R.layout.hotspot_login);

        // bind controls
        mStatus = (TextView) findViewById(R.id.lbStatus);
        mLogin = (EditText) findViewById(R.id.login);
        mPW = (EditText) findViewById(R.id.pw);
        mLockWifi = (CheckBox) findViewById(R.id.cbLockWifi);
        mCreateSSID = (CheckBox) findViewById(R.id.cbCreateSSID);
        mBeep = (CheckBox) findViewById(R.id.cbBeep);
        mScanIntervall = (Spinner) findViewById(R.id.spScanIntervall);
        mHideWaitingIcon = (CheckBox) findViewById(R.id.cbHideIcon);
        mAutostartService = (CheckBox) findViewById(R.id.cbAutostartService);
        mEnableLogCat = (CheckBox) findViewById(R.id.cbEnableLogCat);
        
		serviceBound = false;

		// wait until service is bound and values restored
		valuesRestored = false;
		initializationPhase = true;
		mLogin.setEnabled(false);
		mPW.setEnabled(false);
		mLockWifi.setEnabled(false);
		mCreateSSID.setEnabled(false);
		mBeep.setEnabled(false);
        mScanIntervall.setEnabled(false);
        mHideWaitingIcon.setEnabled(false);
        mAutostartService.setEnabled(false);
        mEnableLogCat.setEnabled(false);
        
        mStatus.setText("Status: unknown");  

        // find buttons in view
        btStart = ((Button) findViewById(R.id.btStart));
        btStop = ((Button) findViewById(R.id.btStop));

        // set actions for buttons
        btStart.setOnClickListener(StartServiceListener);
        btStop.setOnClickListener(StopServiceListener);

        // connect to service
        bindHsliService();
        
        startUpdateThread();

    	SimplePersistence persist = new SimplePersistence(this, "HSLI");
    	String version = persist.getString("version", "0");
    	if (!VERSION.equals(version)) {
    		showDialog(SPLASH_SCREEN);
    	}
        
    }

	private void startUpdateThread() {
		synchronized (HotspotLoginActivity.this) {
			if (updateThread == null) {
				updateThread = new Thread(null, mUpdateTask, "HotspotLoginActivity.UpdateTask");
				updateThread.start();
			}
		}
	}
	private void stopUpdateThread() {
		synchronized (HotspotLoginActivity.this) {
			if (updateThread != null) {
				synchronized (updateThread) {
					updateThread.notify();
				}
				updateThread = null; 
			}
		}
	}

	private void unbindHsliService() {
		if (serviceBound) {
			try {
		        Log.i(TAG, "unbindHsliService-UNBIND");
				unbindService(mHotspotLoginServiceConnection);
				serviceBound = false;
				mHotspotLoginService = null;
			}
			catch (Exception e) {
				Log.e(TAG, e.getMessage(), e);
			}
		}
	}
	
	private void bindHsliService() {
		if (!serviceBound) {
			try {
			    Log.i(TAG, "BIND Service");
				bindService(new Intent(IHotspotLoginService.class.getName()), mHotspotLoginServiceConnection, Context.BIND_AUTO_CREATE);
				serviceBound = true;
			}
			catch (Exception e) {
				Log.e(TAG, e.getMessage(), e);
			}
		}
	}

    
    
    /**
     * Called when your activity's options menu needs to be created.
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        menu.add("Release Notes");
        menu.add("Settings");
        menu.add("Monitor");
        return true;
    }
    
    /**
     * Called when a menu item is selected.
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
    	if (item.getTitle().equals("Release Notes")) {
    		showDialog(SPLASH_SCREEN);
    	}
    	else if (item.getTitle().equals("Settings")) {
	    	IHotspotLoginService hls = mHotspotLoginService;
	    	if (hls == null)
	    		return false;
            stopUpdateThread();
	    	Intent intent = new Intent(this, HotspotLoginSettingsActivity.class)
					.putExtra(EXTRA_SETTINGS, jsonParams);
    		unbindHsliService();
	        startActivityForResult(intent, 1);
    	}
    	else if (item.getTitle().equals("Monitor")) {
            stopUpdateThread();
    		unbindHsliService();
	    	Intent intent = new Intent(this, HotspotLoginMonitorActivity.class)
	    			.putExtra(EXTRA_SETTINGS, jsonParams)
					.putExtra(EXTRA_LOGIN, mLogin.getText().toString())
					.putExtra(EXTRA_PW, mPW.getText().toString());
	        startActivity(intent);
    	}
        return true;
    }
    
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
    	if (resultCode == RESULT_OK) {
    		jsonParams = data.getStringExtra(EXTRA_SETTINGS);
    	}
    	super.onActivityResult(requestCode, resultCode, data);
    }
    
    /**
     * Called when the activity is about to start interacting with the user.
     */
    @Override
    protected void onResume() {
        super.onResume();
        bindHsliService();
        startUpdateThread();
    }

    
    /**
     * A call-back for when the user presses the back button.
     */
    OnClickListener ExitListener = new OnClickListener() {
        public void onClick(View v) {
            finish();
        }
    };

    
    OnClickListener StartServiceListener = new OnClickListener() {
        public void onClick(View v) {
            try {
	        	IHotspotLoginService hsliSvc = mHotspotLoginService;
	        	if (hsliSvc == null) {
	                Toast.makeText(HotspotLoginActivity.this, "Hotspot Login Service not found", Toast.LENGTH_SHORT).show();
	           		return;
	        	}
	        	saveValues();
	        	saveSettings();
	            startService(new Intent(HotspotLoginActivity.this, HotspotLogin_Service.class));
				hsliSvc.start();
			} catch (RemoteException e) {
				Log.e(TAG, e.getMessage(), e);
                Toast.makeText(HotspotLoginActivity.this, "Start Service failed: "+e.getMessage(), Toast.LENGTH_SHORT).show();
			}
        }
    };

    
    
    OnClickListener StopServiceListener = new OnClickListener() {
        public void onClick(View v) {
        	try {
	        	IHotspotLoginService hsliSvc = mHotspotLoginService;
	        	if (hsliSvc == null) {
	                Toast.makeText(HotspotLoginActivity.this, "Hotspot Login Service not found", Toast.LENGTH_SHORT).show();
	           		return;
	        	}
	        	// TODO: logout
	       		hsliSvc.stop();
	            stopService(new Intent(HotspotLoginActivity.this, HotspotLogin_Service.class));
			} catch (RemoteException e) {
				Log.e(TAG, e.getMessage(), e);
	            Toast.makeText(HotspotLoginActivity.this, "Stop Service failed: "+e.getMessage(), Toast.LENGTH_SHORT).show();
			}
       	}
    };


     

	private IHotspotLoginService mHotspotLoginService;
    /**
     * Class for interacting with the service.
     */
    private ServiceConnection mHotspotLoginServiceConnection = new ServiceConnection() {
        public void onServiceConnected(ComponentName className, IBinder service) {
            Log.i(TAG, "BIND.onSvcConn");
            mHotspotLoginService = IHotspotLoginService.Stub.asInterface(service);
        }

        public void onServiceDisconnected(ComponentName className) {
            Log.i(TAG, "BIND.onSvcDisconn");
            mHotspotLoginService = null;
        }
    };


    
    @Override
    protected void onPause() {
        super.onPause();
    }
	
	@Override
    protected void onStop() {
    	super.onStop();
    	stopUpdateThread();
		saveValues();
    	saveSettings();
        // Detach service connection
		unbindHsliService();
    }

	private void saveSettings() {
		if (!valuesRestored)
			return;
		IHotspotLoginService hls = mHotspotLoginService;
		if (hls == null)
			return;
		try {
			if (jsonParams != null) 
				hls.setSettingParams(jsonParams);
		} catch (RemoteException e) {
			Log.e(TAG, e.getMessage(), e);
		}
	}
	
	private void saveValues() {
		if (!valuesRestored)
			return;
		IHotspotLoginService hls = mHotspotLoginService;
		if (hls == null)
			return;
		try {
			hls.setLoginParams(mLogin.getText().toString(), mPW.getText().toString(), mLockWifi.isChecked(), mCreateSSID.isChecked(), mBeep.isChecked(), getScanIntervallSeconds(), mHideWaitingIcon.isChecked(), mAutostartService.isChecked(), mEnableLogCat.isChecked());
		} catch (RemoteException e) {
			Log.e(TAG, e.getMessage(), e);
		}
	}
	private int[] scanIntervallSeconds = {-1, 1*60, 2*60, 5*60, 10*60, 30*60};
	private int getScanIntervallSeconds() {
		int intervallId = mScanIntervall.getSelectedItemPosition();
		if (intervallId == AdapterView.INVALID_POSITION) {
			intervallId = 0;
		}
		return scanIntervallSeconds [intervallId];
	}
    private int getScanIntervallPosition(int seconds) {
    	int result = 0;
    	for (int i=1; i<scanIntervallSeconds.length; i++) {
    		if (scanIntervallSeconds[i]<=seconds) {
    			result = i;
    		}
    	}
		return result;
	}

	private void restoreValues() {
		IHotspotLoginService hls = mHotspotLoginService;
		if (hls == null)
			return;
		try {
			mLogin.setEnabled(true);
	        mLogin.setText(hls.getLogin());
	        String encPW = hls.getEncryptedPW();
			mPW.setEnabled(true);
	        mPW.setText(SimpleCrypto.decrypt(TRANSMIT_SEED, encPW));
			mLockWifi.setEnabled(true);
	        mLockWifi.setChecked(hls.getWifiLockEnabled());
			mCreateSSID.setEnabled(true);
	        mCreateSSID.setChecked(hls.getAutoCreateConfigForSSID());
			mBeep.setEnabled(true);
			mBeep.setChecked(hls.getBeepEnabled());
			mScanIntervall.setEnabled(true);
			mScanIntervall.setSelection(getScanIntervallPosition(hls.getScanIntervallSeconds()));
			mHideWaitingIcon.setEnabled(true);
			mHideWaitingIcon.setChecked(hls.getHideWaitingIcon());
			mAutostartService.setEnabled(true);
			mAutostartService.setChecked(hls.getAutostartService());
			mEnableLogCat.setEnabled(true);
			mEnableLogCat.setChecked(hls.getEnableLogCat());
			
			jsonParams = hls.getJsonParams();
	        valuesRestored = true;
		} catch (Exception e) {
			Log.e(TAG, e.getMessage(), e);
		}
	}



	Runnable mUpdateTask = new Runnable() {
        public void run() {
        	try {
        		Log.i(TAG+".updatetask", "updateTask is running");
        		Thread thr = Thread.currentThread();
	        	while (updateThread == thr) {
	        		try {
		        		synchronized (thr) {
			        		thr.wait(1000);
						}
		        		IHotspotLoginService hls = mHotspotLoginService;
		        		if (hls != null) {
	        				if (!valuesRestored)
	        					uiHandler.restValues();
		        			if (hls.isRunning()) {
		        				uiHandler.setStatus("Status: " + hls.getCurrentHotspotStatusName());
		        			}
		        			else
		        				uiHandler.setStatus("Status: Service not started");
		        		}
		        		else
		        			uiHandler.setStatus("Status: not bound to service");
		        	}
	        		catch (Exception e) {
	            		Log.e(TAG+".updatetask", e.getMessage(), e);
					}
	        	}
        	}
        	catch (Exception e) {
        		Log.e(TAG+".updatetask", e.getMessage(), e);
        	}
    		Log.i(TAG+".updatetask", "finished updateTask");
        }
    };

    
    @Override
    protected Dialog onCreateDialog(int id) {
        switch (id) {
        case SPLASH_SCREEN:
	        LayoutInflater factory = LayoutInflater.from(this);
	        final View splashView = factory.inflate(R.layout.splash_screen, null);
			final TextView tvReleaseNotes = (TextView)splashView.findViewById(R.id.release_text);
			String release_notes = Helper.readRawResource(this, "release_notes");
			if (release_notes != null) {
				release_notes = release_notes.replaceAll("\\r\\n", "\n");
				tvReleaseNotes.setText(release_notes);
			}
	        return new AlertDialog.Builder(this)
	            .setIcon(R.drawable.hotspot_login_icon)
	            .setTitle("New in Release "+VERSION)
	            .setView(splashView)
	            .setPositiveButton("OK", new DialogInterface.OnClickListener() {
	                public void onClick(DialogInterface dialog, int whichButton) {
	    				try {
	    					SimplePersistence persist = new SimplePersistence(HotspotLoginActivity.this, "HSLI");
		                	persist.putString("version", VERSION);
		                	persist.commit();
	    				} catch (Exception e) {
	    					Log.e(TAG, "Reset failed: "+e.getMessage(), e);
	    				}
	                }
	            })
	            .create();
        }
	    return null;
    }
    
}
