/*******************************************************************************
 * Copyright (c) 2009 Ferenc Hechler - ferenc_hechler@users.sourceforge.net
 * 
 * This file is part of the Android Hotspot Login
 *
 * The Android Hotspot Login is free software;
 * you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option) any later version.
 * 
 * The Android Hotspot Login is distributed
 * in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the Android Hotspot Login;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *  
 *******************************************************************************/
package net.sf.andhsli.hotspotlogin;

import java.util.List;

import net.sf.andhsli.hotspotlogin.HotspotUtilities.HotspotStatus;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.wifi.ScanResult;
import android.net.wifi.SupplicantState;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

/**
 * This class provides a basic demonstration of how to write an Android
 * activity. Inside of its window, it places a single view: an EditText that
 * displays and edits some internal text.
 */
public class HotspotLoginMonitorActivity extends Activity {

	private final static String TAG = "HSLI.monitor";
	
    enum Action {SLEEP, SCAN, STATUS, CONNECT, LOCK, DISCONNECT, LOGIN, START_SERVICE, STOP_SERVICE, LOGOUT, EXIT};
    private final static int MSG_LOG = 1;
    private final static int MSG_CLEARLOG = 2;
    private final static int MSG_SETSTATUS = 3;
    private final static int MSG_DISABLEBUTTONS = 4;
    private final static int MSG_ENABLEBUTTONS = 5;

    public final static String PREFS_NAME = "HotspotLoginPrefs";
	public final static String SETTINGPREFS_NAME = "HotspotLoginSettings3Prefs";

    
    private Params params;
    private String login;
    private String pw;
    
    class UIHandler extends Handler {
    	@Override
    	public void handleMessage(Message msg) {
    		super.handleMessage(msg);
    		switch (msg.what) {
        		case(MSG_LOG): {
        			String txt = (String)msg.obj;
    		   		mOutput.setText(mOutput.getText().toString()+txt+"\n");
    				Log.i(TAG, txt);
        			break;
        		}
        		case(MSG_CLEARLOG): {
    				mOutput.setText("");
        			break;
        		}
        		case(MSG_SETSTATUS): {
        			String st = (String)msg.obj;
        			mStatus.setText(st);
        			break;
        		}
        		case(MSG_DISABLEBUTTONS): {
	        		btScan.setEnabled(false);
	        		btStatus.setEnabled(false);
	        		btConnect.setEnabled(false);
	        		btDisconnect.setEnabled(false);
	        		btLogin.setEnabled(false);
	        		btLogout.setEnabled(false);
	    			break;
	    		}
        		case(MSG_ENABLEBUTTONS): {
	        		btScan.setEnabled(true);
	        		btStatus.setEnabled(true);
	        		btConnect.setEnabled(true);
	        		btDisconnect.setEnabled(true);
	        		btLogin.setEnabled(true);
	        		btLogout.setEnabled(true);
	    			break;
	    		}
    		}
    	}
    	public void log(String txt) {
    		Message msg = Message.obtain(this, MSG_LOG, txt);
    		sendMessage(msg);
    	}
    	public void clearLog() {
    		Message msg = Message.obtain(this, MSG_CLEARLOG);
    		sendMessage(msg);
    	}
    	public void setStatus(String st) {
    		Message msg = Message.obtain(this, MSG_SETSTATUS, st);
    		sendMessage(msg);
    	}
    	public void disableButtons() {
    		Message msg = Message.obtain(this, MSG_DISABLEBUTTONS);
    		sendMessage(msg);
    	}
    	public void enableButtons() {
    		Message msg = Message.obtain(this, MSG_ENABLEBUTTONS);
    		sendMessage(msg);
    	}
	};
	
	private TextView mStatus;
	private EditText mOutput;
	
	Button btScan;
	Button btStatus;
	Button btConnect;
	Button btDisconnect;
	Button btLogin;
	Button btLogout;
	Button btExit;
	
	private UIHandler uiHandler;
	private Handler execHandler;
	
	private Action currentAction;
	
	public HotspotLoginMonitorActivity() {
    }

    /** Called with the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        uiHandler = new UIHandler();
        new Thread() {
            public void run() {
                Looper.prepare();
                execHandler = new Handler();
                Looper.loop();
            }
		}.start();
		while (execHandler == null)
			try { Thread.sleep(50); } catch (InterruptedException ignore){};
		currentAction = Action.SLEEP;
		
		// create view from xml
        setContentView(R.layout.hotspot_login_monitor);

        // bind controls
        mStatus = (TextView) findViewById(R.id.lbStatus);
        mOutput= (EditText) findViewById(R.id.output);

        mStatus.setText("UNKNOWN");  

        // find buttons in view
        btScan = ((Button) findViewById(R.id.btScan));
        btStatus = ((Button) findViewById(R.id.btStatus));
        btConnect = ((Button) findViewById(R.id.btConnect));
        btDisconnect = ((Button) findViewById(R.id.btDisconnect));
        
        btLogin = ((Button) findViewById(R.id.btLogin));
        btLogout = ((Button) findViewById(R.id.btLogout));
        
        btExit = ((Button) findViewById(R.id.btExit));

        // set actions for buttons
        btScan.setOnClickListener(new ActionListener(Action.SCAN, ScanAction));
        btStatus.setOnClickListener(new ActionListener(Action.STATUS, StatusAction));
        btConnect.setOnClickListener(new ActionListener(Action.CONNECT, ConnectAction));
        btDisconnect.setOnClickListener(new ActionListener(Action.DISCONNECT, DisconnectAction));
        
        btLogin.setOnClickListener(new ActionListener(Action.LOGIN, LoginAction));
        btLogout.setOnClickListener(new ActionListener(Action.LOGOUT, LogoutAction));
        
        btExit.setOnClickListener(ExitListener);

        // Restore preferences
        Intent data = getIntent();
        params = new Params();
        String jsonParams = data.getStringExtra(HotspotLoginActivity.EXTRA_SETTINGS);
		try {
			params.fromJSON(jsonParams); 
    	}
    	catch (Exception e) {
    		uiHandler.clearLog();
    		uiHandler.log("invalid settings (json format): "+e.getMessage());
    		Log.e(TAG, e.getMessage(), e);
		}
    	login = data.getStringExtra(HotspotLoginActivity.EXTRA_LOGIN);
		pw = data.getStringExtra(HotspotLoginActivity.EXTRA_PW);
        
        mOutput.setText(getText(R.string.welcome_text));
    }
    
    /**
     * Called when the activity is about to start interacting with the user.
     */
    /**
     * A call-back for when the user presses the back button.
     */
    OnClickListener ExitListener = new OnClickListener() {
        public void onClick(View v) {
            finish();
        }
    };

    
    private class ActionListener implements OnClickListener {
    	Action action;
    	ActionExecutable executable;
    	ActionListener(Action action, ActionExecutable executable) {
    		this.action = action;
    		this.executable = executable;
    	}
        public void onClick(View v) {
        	if (currentAction == Action.SLEEP) {
        		currentAction = action;
        		uiHandler.clearLog();
        		uiHandler.log("starting action " + action.toString());
        		execHandler.post(executable);
        	}
        }
     };
    
     abstract class ActionExecutable implements Runnable {
    	 abstract void exec() throws Exception;
    	 @Override
    	public void run() {
    		try {
    			uiHandler.disableButtons();
    			exec();
    		}
    		catch (Exception e) {
    			uiHandler.log("EXCEPTION: " + e.getMessage());
    			Log.e(TAG+","+currentAction.toString(), e.getMessage(), e);
    			StackTraceElement[] st = e.getStackTrace();
    			for (int i = 0; i < st.length; i++) {
        			uiHandler.log(st[i].toString());
				}
			}
    		finally {
    			currentAction = Action.SLEEP;
    			uiHandler.enableButtons();
    		}
    	}
     }
     
     ActionExecutable ScanAction = new ActionExecutable() {
         public void exec() throws Exception {
         	 WifiManager wifiMan = (WifiManager)getSystemService(Context.WIFI_SERVICE);
       	     List<ScanResult> srList = wifiMan.getScanResults();
       	     boolean start = wifiMan.startScan();
   		     uiHandler.log("SCANLIST " + start);
       	     if (srList != null) {
          	     for (ScanResult sr : srList) {
          		     uiHandler.log(sr.toString());
          	     }
       	    }
         }
     };
     
    ActionExecutable StatusAction = new ActionExecutable() {
        public void exec() throws Exception {
			uiHandler.log("getting WifiManager");
      		WifiManager wifiMan = (WifiManager)getSystemService(Context.WIFI_SERVICE);
			uiHandler.log("getting ConnectivityManager");
        	ConnectivityManager conyMan = (ConnectivityManager)getSystemService(CONNECTIVITY_SERVICE); 
      		HotspotStatus hsStatus = HotspotUtilities.getHotspotStatus(params, wifiMan, conyMan);
        	uiHandler.setStatus(hsStatus.toString());
        	
			uiHandler.log("getting WifiInfo");
 			WifiInfo conInfo= wifiMan.getConnectionInfo();
			if (conInfo == null) {
				uiHandler.log("WifiInfo is NULL");
				uiHandler.log("STATUS: -");
				return;
			}
			uiHandler.log("WifiInfo found: '"+conInfo.toString()+"'");
			if (!params.hasSsid(conInfo.getSSID())) {
				uiHandler.log("wrong SSID selected: '"+conInfo.getSSID()+"'");
				uiHandler.log("STATUS: w");
				return;
			}
			params.setCurrentSsid(conInfo.getSSID());
			uiHandler.log("SSID "+params.getSSID()+" found: '"+conInfo.getSupplicantState()+"'");
			if (conInfo.getSupplicantState() != SupplicantState.COMPLETED) {
				uiHandler.log("connection not completed: '"+conInfo.getSupplicantState()+"'");
				uiHandler.log("STATUS: ~");
				return;
        	}
			uiHandler.log("connection completed: '"+conInfo.getSupplicantState()+"'");
			uiHandler.log("querying " + params.getStatusCheckUrl());
			String response = HotspotUtilities.httpGet(params.getStatusCheckUrl(), params.getUserAgent());
			if (response.indexOf(params.getStatusCheckSucceededResponse()) != -1) {
				uiHandler.log("found check succeeded response: '"+params.getLoggedInText()+"'");
				uiHandler.log("STATUS: +");
				uiHandler.log(".\n.\nHTTP-RESPONSE: " + response);
				return;
			}
			if (response.indexOf(params.getStatusCheckFailedResponse()) != -1) {
				uiHandler.log("found check failed response: '"+params.getLoggedInText()+"'");
				uiHandler.log("STATUS: o");
				uiHandler.log(".\n.\nHTTP-RESPONSE: " + response);
				return;
			}
			uiHandler.log("log in not found: '"+conInfo.getSupplicantState()+"'");
			uiHandler.log("STATUS: ?");
			uiHandler.log(".\n.\nHTTP-RESPONSE: " + response);
         }
     };


     ActionExecutable ConnectAction = new ActionExecutable() {
         public void exec() {
     		WifiManager wifiMan = (WifiManager)getSystemService(Context.WIFI_SERVICE);
       	    List<ScanResult> srList = wifiMan.getScanResults();
       	    boolean start = wifiMan.startScan();
       	    uiHandler.log("SCANLIST " + start);
       	    if (srList != null) {
       	    	uiHandler.log("#scans=" + srList.size());
          	    for (ScanResult sr : srList) {
          	    	uiHandler.log(sr.toString());
          	    	if (params.hasSsid(sr.SSID)) {
          	    		params.setCurrentSsid(sr.SSID);
          			    List<WifiConfiguration> confNets = wifiMan.getConfiguredNetworks();
          			    uiHandler.log("get configured nets #" + confNets.size());
          			    for (WifiConfiguration confNet:confNets) {
          				    uiHandler.log("configured net " + confNet.SSID);
          				    if (("\""+sr.SSID+"\"").equals(confNet.SSID)) {
          					    boolean result = wifiMan.enableNetwork(confNet.networkId, true);
          					    uiHandler.log("ENABLING configured net" + confNet.networkId + " = " + result);
          				    }
          				    else {
          					    uiHandler.log("unequal('"+sr.SSID+"','"+confNet.SSID+"')");
          				    }
          			    }
          			    // TODO: check for SSID is not configured, create configuration
            		}
          	    }
       	    }
         }
     };
     

     ActionExecutable DisconnectAction = new ActionExecutable() {
         public void exec() {
     		WifiManager wifiMan = (WifiManager)getSystemService(Context.WIFI_SERVICE);
 			WifiInfo conInfo= wifiMan.getConnectionInfo();
 			uiHandler.log("WIFIINFO: " + conInfo.toString());
 			if (conInfo != null) {
 				uiHandler.log("SSID='" + conInfo.getSSID() + "'");
 				if (params.hasSsid(conInfo.getSSID())) {
 					params.setCurrentSsid(conInfo.getSSID());
 	 				uiHandler.log("SupplicantState='" + conInfo.getSupplicantState() + "'");
 					if (conInfo.getSupplicantState() == SupplicantState.COMPLETED) {
 						boolean result = wifiMan.disableNetwork(conInfo.getNetworkId());
 						uiHandler.log("DISABLENETWORK: " + result);
 					}
 				}
     		}    			
         }
     };
     
     
    ActionExecutable LoginAction = new ActionExecutable() {
        public void exec() throws Exception {
            uiHandler.log("login="+login);
            String logInURL = params.getLogInURL();
            String logInParams = params.getLogInParams();
            String[] paramList = logInParams.split("[|]");
            String[][] logInAttribs = new String[paramList.length][2];
            for (int i = 0; i < paramList.length; i++) {
            	logInAttribs[i][0] = paramList[i].replaceFirst("^([^=]*)[=](.*)$", "$1");
            	logInAttribs[i][1] = paramList[i].replaceFirst("^([^=]*)[=](.*)$", "$2");
            	logInAttribs[i][1] = HotspotUtilities.replaceAllFix(logInAttribs[i][1] , "$LOGIN", login);
            	logInAttribs[i][1] = HotspotUtilities.replaceAllFix(logInAttribs[i][1] , "$PW", pw);
			}
            String response = HotspotUtilities.httpPost(logInURL, logInAttribs, params.getUserAgent());
            if (response != null) {
            	if (response.indexOf(params.getLoggedInText())!=-1) {
            		uiHandler.log("LoggedInText found");
            	}
            	if (response.indexOf(params.getLoggedOutText())!=-1) {
            		uiHandler.log("LoggedOutText found");
            	}
            }
        	uiHandler.log("RESPONSE: " + response);
        }
    };

     
     ActionExecutable LogoutAction = new ActionExecutable() {
         public void exec() throws Exception {
           String result = HotspotUtilities.httpGet(params.getLogOutURL(), params.getUserAgent());
           uiHandler.log(result);
         };
     };
    
	
	@Override
    protected void onStop() {
    	super.onStop();
    }

}
