/*******************************************************************************
 * Copyright (c) 2009 Ferenc Hechler - ferenc_hechler@users.sourceforge.net
 * 
 * This file is part of the Android Hotspot Login
 *
 * The Android Hotspot Login is free software;
 * you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option) any later version.
 * 
 * The Android Hotspot Login is distributed
 * in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the Android Hotspot Login;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *  
 *******************************************************************************/
package net.sf.andhsli.hotspotlogin;

import org.json.JSONException;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

/**
 * This class provides a basic demonstration of how to write an Android
 * activity. Inside of its window, it places a single view: an EditText that
 * displays and edits some internal text.
 */
public class HotspotLoginSettingsActivity extends Activity {

	private EditText mParams;
	
	Button btReset;
	Button btCancel;
	Button btOK;
	
	public HotspotLoginSettingsActivity() {
    }

    /** Called with the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

		// create view from xml
        setContentView(R.layout.paramsettings);

        // bind controls
        mParams = (EditText) findViewById(R.id.paramsText);

        // find buttons in view
        btReset = ((Button) findViewById(R.id.btReset));
        btCancel = ((Button) findViewById(R.id.btCancel));
        btOK = ((Button) findViewById(R.id.btOK));

        // set actions for buttons
        btReset.setOnClickListener(ResetListener);
        btCancel.setOnClickListener(CancelListener);
        btOK.setOnClickListener(OKListener);

        // Restore preferences
        String jsonText = getIntent().getStringExtra(HotspotLoginActivity.EXTRA_SETTINGS);
        try {
			Params params = new Params(); 
			params.fromJSON(jsonText);
			mParams.setText(params.toJSON(1));
    	}
    	catch (Exception e) {
            Toast.makeText(HotspotLoginSettingsActivity.this, "invalid json format: "+e.getMessage(), Toast.LENGTH_SHORT).show();
    		Log.e("HSLI.set", e.getMessage(), e);
		}
    }

    
    
    /**
     * Called when the activity is about to start interacting with the user.
     */
    @Override
    protected void onResume() {
        super.onResume();
    }


    /**
     * A call-back for when the user presses the back button.
     */
    OnClickListener CancelListener = new OnClickListener() {
        public void onClick(View v) {
    		setResult(RESULT_CANCELED);
            finish();
        }
    };

    OnClickListener ResetListener = new OnClickListener() {
        public void onClick(View v) {
        	try {
//        		String jsonText="[{" +
//        				"\"SSID\":\"tmobile\"," +
//        				"\"userAgent\":\"Android Hotspot Login v0.0.3\"," +
//        				"\"statusCheckUrl\":\"http://google.de\"," +
//        				"\"statusCheckFailedResponse\":\"https://hotspot.t-mobile.net\"," +
//        				"\"statusCheckSucceededResponse\":\"http://www.google.de\"," +
//        				"\"logInURL\":\"https://hotspot.t-mobile.net/wlan/index.do\"," +
//        				"\"logInParams\":\"username=$LOGIN|password=$PW|strHinweis=Zahlungsbedingungen|strAGB=AGB\"," +
//        				"\"loggedInText\":\"Sie sind online.\"," +
//        				"\"loggedOutText\":\"Login wurde abgelehnt.\"," +
//        				"\"logOutURL\":\"https://hotspot.t-mobile.net/wlan/goodbye.do\"," +
//						"\"waitForSsidPollIntervall\":\"60\"," +
//						"\"internetPollIntervall\":\"60\"," +
//        				"\"shortWait\":\"5\"," +
//        				"\"mediumWait\":\"15\"," +
//        				"\"longWait\":\"60\"" +
//        		"}]";

        		Params params = new Params();
//        		params.fromJSON(jsonText);
				mParams.setText(params.toJSON(1));
			} catch (JSONException e) {
				Log.e("HSLI", e.getMessage(), e);
			}
        }
    };
    OnClickListener OKListener = new OnClickListener() {
        public void onClick(View v) {
            // save settings
        	Params params = new Params();
        	String jsonText = mParams.getText().toString();
            Log.e("HSLI.set", "OK-params="+jsonText);
        	try {
        		params.fromJSON(jsonText);
        		String formatedJsonText = params.toJSON(0);
        		setResult(RESULT_OK, new Intent().putExtra(HotspotLoginActivity.EXTRA_SETTINGS, formatedJsonText));
        		finish();
        	}
        	catch (Exception e) {
                Toast.makeText(HotspotLoginSettingsActivity.this, "invalid json format: "+e.getMessage(), Toast.LENGTH_SHORT).show();
        		Log.e("HSLI.set", e.getMessage(), e);
			}
        }
    };

    
    @Override
    protected void onPause() {
        super.onPause();
    }
	
	@Override
    protected void onStop() {
    	super.onStop();
    }
	
	
	
	
}
